@@ -1,19 +1,21 @@
+from dataclasses import replace
 from datetime import datetime, timedelta
 from functools import total_ordering
 from typing import Iterable, List, Optional, Tuple, Union, cast
 
 import pandas as pd
 import strawberry
+from strawberry import UNSET
 
-from phoenix.core.model_schema import CONTINUOUS, PRIMARY, REFERENCE, Dimension, Model
+from phoenix.core.model_schema import CONTINUOUS, PRIMARY, REFERENCE, Column, Dimension, Inferences
 from phoenix.metrics import Metric, binning
-from phoenix.metrics.mixins import DriftOperator
+from phoenix.metrics.mixins import UnaryOperator
 from phoenix.metrics.timeseries import timeseries
 from phoenix.server.api.input_types.Granularity import Granularity, to_timestamps
 from phoenix.server.api.input_types.TimeRange import TimeRange
-from phoenix.server.api.interceptor import NoneIfNan
-from phoenix.server.api.types import METRICS
+from phoenix.server.api.interceptor import GqlValueMediator
 from phoenix.server.api.types.DataQualityMetric import DataQualityMetric
+from phoenix.server.api.types.InferencesRole import InferencesRole
 from phoenix.server.api.types.ScalarDriftMetricEnum import ScalarDriftMetric
 from phoenix.server.api.types.VectorDriftMetricEnum import VectorDriftMetric
 
@@ -27,14 +29,16 @@ class TimeSeriesDataPoint:
     timestamp: datetime
 
     """The value of the data point"""
-    value: Optional[float] = strawberry.field(default=NoneIfNan())
+    value: Optional[float] = strawberry.field(default=GqlValueMediator())
 
-    def __lt__(self, other: "TimeSeriesDataPoint") -> bool:
+    def __lt__(self, other: "TimeSeriesDataPoint") -> bool:  # type: ignore
         return self.timestamp < other.timestamp
 
 
 def to_gql_datapoints(
-    df: pd.DataFrame, metric: Metric, timestamps: Iterable[datetime]
+    df: pd.DataFrame,
+    metric: Metric,
+    timestamps: Iterable[datetime],
 ) -> List[TimeSeriesDataPoint]:
     data = []
     for timestamp in timestamps:
@@ -58,24 +62,12 @@ class TimeSeries:
     data: List[TimeSeriesDataPoint]
 
 
-def _get_timeseries_data(
-    dimension: Dimension,
-    metric: Union[ScalarDriftMetric, VectorDriftMetric, DataQualityMetric],
+def get_timeseries_data(
+    df: pd.DataFrame,
+    metric: Metric,
     time_range: TimeRange,
     granularity: Granularity,
 ) -> List[TimeSeriesDataPoint]:
-    if not (metric_cls := METRICS.get(metric.value, None)):
-        raise NotImplementedError(f"Metric {metric} is not implemented.")
-    data = dimension[PRIMARY]
-    metric_instance = metric_cls(operand_column_name=dimension.name)
-    if issubclass(metric_cls, DriftOperator):
-        ref_data = dimension[REFERENCE]
-        metric_instance.reference_data = pd.DataFrame({dimension.name: ref_data})
-        if dimension.data_type is CONTINUOUS:
-            metric_instance.binning_method = binning.QuantileBinning(
-                reference_series=dimension[REFERENCE],
-            )
-    df = pd.DataFrame({dimension.name: data})
     return df.pipe(
         timeseries(
             start_time=time_range.start,
@@ -87,10 +79,10 @@ def _get_timeseries_data(
                 minutes=granularity.sampling_interval_minutes,
             ),
         ),
-        metrics=(metric_instance,),
+        metrics=(metric,),
     ).pipe(
         to_gql_datapoints,
-        metric=metric_instance,
+        metric=metric,
         timestamps=to_timestamps(time_range, granularity),
     )
 
@@ -105,8 +97,24 @@ def get_data_quality_timeseries_data(
     metric: DataQualityMetric,
     time_range: TimeRange,
     granularity: Granularity,
+    inferences_role: InferencesRole,
 ) -> List[TimeSeriesDataPoint]:
-    return _get_timeseries_data(dimension, metric, time_range, granularity)
+    metric_instance = metric.value()
+    if isinstance(metric_instance, UnaryOperator):
+        metric_instance = replace(
+            metric_instance,
+            operand=Column(dimension.name),
+        )
+    df = pd.DataFrame(
+        {dimension.name: dimension[inferences_role.value]},
+        copy=False,
+    )
+    return get_timeseries_data(
+        df,
+        metric_instance,
+        time_range,
+        granularity,
+    )
 
 
 @strawberry.type
@@ -119,19 +127,47 @@ def get_drift_timeseries_data(
     metric: Union[ScalarDriftMetric, VectorDriftMetric],
     time_range: TimeRange,
     granularity: Granularity,
+    reference_data: pd.DataFrame,
 ) -> List[TimeSeriesDataPoint]:
-    return _get_timeseries_data(dimension, metric, time_range, granularity)
+    metric_instance = metric.value()
+    metric_instance = replace(
+        metric_instance,
+        operand=Column(dimension.name),
+        reference_data=reference_data,
+    )
+    if isinstance(metric, ScalarDriftMetric) and dimension.data_type is CONTINUOUS:
+        metric_instance = replace(
+            metric_instance,
+            binning_method=binning.QuantileBinning(
+                reference_series=dimension[REFERENCE],
+            ),
+        )
+    df = pd.DataFrame(
+        {dimension.name: dimension[PRIMARY]},
+        copy=False,
+    )
+    return get_timeseries_data(
+        df,
+        metric_instance,
+        time_range,
+        granularity,
+    )
+
+
+@strawberry.type
+class PerformanceTimeSeries(TimeSeries):
+    """A time series of drift metrics"""
 
 
 def ensure_timeseries_parameters(
-    model: Model,
-    time_range: Optional[TimeRange] = None,
-    granularity: Optional[Granularity] = None,
+    inferences: Inferences,
+    time_range: Optional[TimeRange] = UNSET,
+    granularity: Optional[Granularity] = UNSET,
 ) -> Tuple[TimeRange, Granularity]:
-    if time_range is None:
-        start, end = model[PRIMARY].time_range
-        time_range = TimeRange(start=start, end=end)
-    if granularity is None:
+    if not isinstance(time_range, TimeRange):
+        start, stop = inferences.time_range
+        time_range = TimeRange(start=start, end=stop)
+    if not isinstance(granularity, Granularity):
         total_minutes = int((time_range.end - time_range.start).total_seconds()) // 60
         granularity = Granularity(
             evaluation_window_minutes=total_minutes,
